Package: transformr
Type: Package
Title: Polygon and Path Transformations
Version: 0.1.5
Authors@R: 
    person(given = "Thomas Lin",
           family = "Pedersen",
           role = c("cre", "aut"),
           email = "thomasp85@gmail.com",
           comment = c(ORCID = "0000-0002-5147-4711"))
Maintainer: Thomas Lin Pedersen <thomasp85@gmail.com>
Description: In order to smoothly animate the transformation of polygons and 
    paths, many aspects needs to be taken into account, such as differing number 
    of control points, changing center of rotation, etc. The 'transformr' 
    package provides an extensive framework for manipulating the shapes of 
    polygons and paths and can be seen as the spatial brother to the 'tweenr' 
    package.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: tweenr, rlang, sf, lpSolve, vctrs
Suggests: covr, magrittr
LinkingTo: cpp11
RoxygenNote: 7.3.1
URL: https://github.com/thomasp85/transformr
BugReports: https://github.com/thomasp85/transformr/issues
NeedsCompilation: yes
Packaged: 2024-02-26 13:46:42 UTC; thomas
Author: Thomas Lin Pedersen [cre, aut]
    (<https://orcid.org/0000-0002-5147-4711>)
Repository: CRAN
Date/Publication: 2024-02-26 14:30:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 05:33:25 UTC; unix
Archs: transformr.so.dSYM
