# Generated by roxygen2: do not edit by hand

export(path_spiral)
export(path_waves)
export(point_grid)
export(point_random)
export(poly_circle)
export(poly_circles)
export(poly_star)
export(poly_star_hole)
export(st_normalize)
export(tween_path)
export(tween_polygon)
export(tween_sf)
import(vctrs)
importFrom(lpSolve,lp.assign)
importFrom(rlang,"%||%")
importFrom(rlang,as_function)
importFrom(rlang,enquo)
importFrom(rlang,eval_tidy)
importFrom(rlang,quo)
importFrom(rlang,quo_is_null)
importFrom(sf,"st_crs<-")
importFrom(sf,"st_geometry<-")
importFrom(sf,st_area)
importFrom(sf,st_as_sfc)
importFrom(sf,st_bbox)
importFrom(sf,st_cast)
importFrom(sf,st_centroid)
importFrom(sf,st_combine)
importFrom(sf,st_crs)
importFrom(sf,st_distance)
importFrom(sf,st_geometry)
importFrom(sf,st_geometry_type)
importFrom(sf,st_intersection)
importFrom(sf,st_length)
importFrom(sf,st_linestring)
importFrom(sf,st_multilinestring)
importFrom(sf,st_multipoint)
importFrom(sf,st_multipolygon)
importFrom(sf,st_point)
importFrom(sf,st_polygon)
importFrom(sf,st_sample)
importFrom(sf,st_sfc)
importFrom(sf,st_touches)
importFrom(sf,st_transform)
importFrom(sf,st_union)
importFrom(sf,st_voronoi)
importFrom(stats,runif)
importFrom(tweenr,.complete_states)
importFrom(tweenr,.get_last_frame)
importFrom(tweenr,.has_frames)
importFrom(tweenr,.max_id)
importFrom(tweenr,.with_prior_frames)
importFrom(tweenr,tween_state)
useDynLib(transformr, .registration = TRUE)
