# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

find_splits <- function(lengths, n) {
    .Call('_transformr_find_splits', PACKAGE = 'transformr', lengths, n)
}

insert_points <- function(x, y, splits, n) {
    .Call('_transformr_insert_points', PACKAGE = 'transformr', x, y, splits, n)
}

rotate <- function(poly_x, poly_y, ref_x, ref_y) {
    .Call('_transformr_rotate', PACKAGE = 'transformr', poly_x, poly_y, ref_x, ref_y)
}

fill_down <- function(nas) {
    .Call('_transformr_fill_down', PACKAGE = 'transformr', nas)
}

unpack_sf <- function(sf, type) {
    .Call('_transformr_unpack_sf', PACKAGE = 'transformr', sf, type)
}

repack_sf <- function(df, type, n_frames) {
    .Call('_transformr_repack_sf', PACKAGE = 'transformr', df, type, n_frames)
}

