# Generated by cpp11: do not edit by hand

find_splits_c <- function(lengths, n) {
  .Call(`_transformr_find_splits_c`, lengths, n)
}

insert_points_c <- function(x, y, splits, n) {
  .Call(`_transformr_insert_points_c`, x, y, splits, n)
}

rotate_c <- function(poly_x, poly_y, ref_x, ref_y) {
  .Call(`_transformr_rotate_c`, poly_x, poly_y, ref_x, ref_y)
}

fill_down <- function(nas) {
  .Call(`_transformr_fill_down`, nas)
}

unpack_sf <- function(sf, type) {
  .Call(`_transformr_unpack_sf`, sf, type)
}

repack_sf <- function(df, type, n_frames) {
  .Call(`_transformr_repack_sf`, df, type, n_frames)
}
